define(['jquery', 'Config', 'CatalogController', 'CleaningController', 'CoverageController',
        'ExternalDataHandler', 'LanguageController', 'LocalStorage', 'ProductDetailController',
        'SeminarController', 'SeminarDetailController', 'TemplateEngine', 'TechnicalRequest',
        'Consultants', 'Scanner', 'AppHistory', 'Eyecatcher', 'ProcedureController', 'PDFDialog'],
        function($, Config, CatalogController, CleaningController, CoverageController,
            ExternalDataHandler, LanguageController, LocalStorage, ProductDetailController,
            SeminarController, SeminarDetailController, TemplateEngine, TechnicalRequest,
            Consultants, Scanner, AppHistory, Eyecatcher, ProcedureController, PDFDialog) {

    'use strict';

    /**
    @class App
    @constructor
    **/
    var App = (function() {

        var iframes,    // Höhe der iframes muss einmalig gesetzt werden
            isDevice,
            isApprove = false,
            strings;    // Sprachpaket

        /**
        App initialisieren. Es werden sämtliche init-Routinen aufgerufen und
        der Start der Anwendung vorgebereitet.

        @method initialise
        @param {Boolean} desktop handelt es sich um ein mobiles Device oder einen Desktop
        **/
        function init(desktop) {

            isDevice = !desktop;

            log('App.init() called', 'Device ist ein mobiles Device: ' + isDevice, Config);

            Config.refreshState = function(link) {
                var item = $(link);

                if (item.hasClass('ui-link-inherit')) {
                    item = item.parents('.ui-btn');
                }

                setTimeout(function() {
                    item.removeClass($.mobile.activeBtnClass);
                }, 100);
            };

            // globale Callbacks für AJAX-Request definieren
            ExternalDataHandler.callbacks = {
                error : function() {
                    $(document).trigger('cAlert', 'error');
                    Config.loader('hide');
                },
                success : null
            };

            initListeners();

            setTimeout(checkAgreement, 50);    // braucht's für die mobilen Devices

        }

        /**
        Beim Start muss geschaut werden, ob der User die Nutzungsbedingungen
        akzeptiert hat und ob eine Sprache gesetzt wurde. Wenn nicht, dann
        wird die Startseite angezeigt, ansonsten geht es zum Homescreen.

        @private
        @method checkAgreement
        **/
        function checkAgreement() {
            if (LocalStorage.get(Config.storage.hasApproved)) {
                Eyecatcher.init();
                LanguageController.check(LocalStorage.get(Config.storage.languageCode));
            } else {
                isApprove = true;
                initLanguageSwitch(true);
                $.mobile.changePage($(Config.pages.language), {
                    changeHash : false,
                    transition : 'fade'
                });
            }
        }

        /**
        Wir klinken uns in den Page-Change Prozess ein, um die entsprechenden
        Templates rendern zu können bzw. die Controller zu steuern. Darüber
        hinaus werden auch CustomEvents behandelt.

        @private
        @method initListeners
        **/
        function initListeners() {

            /*
            CustomEvents müssen mittels **document.addEventListener** abgefangen
            werden, da $(document).on() lediglich das Basis-Event zurück
            liefert.
            */
            // Sprachpaket wurde erfolgreich geladen
            $(document).on('cLanguageLoaded', function(e, param) {
                log('CustomEvent', e, param);

                // Seminare in allen Sprachen außer Deutsch ausblenden
                var footer = $('[data-role=footer] fieldset', Config.pages.home);

                if ('DE' === Config.currentLanguage) {
                    $('.ui-block-b', footer).show();
                    footer.attr('class', 'ui-grid-b');
                } else {
                    $('.ui-block-b', footer).hide();
                    footer.attr('class', 'ui-grid-a');
                }

                strings = LanguageController.getStrings();

                TemplateEngine.translate(strings,
                    null, LocalStorage.get(Config.storage.languageCode));


                if (!LocalStorage.get(Config.storage.hasApproved)) {
                    // $.mobile.changePage($(Config.pages.approve));
                    initApprovingDialog();
                } else {
                    /*
                    Nach einem Wechsel muss der LocalStorage neu befüllt
                    werden. Daher werden die davon betroffenen Daten aus
                    dem LocalStorage entfernt.
                    */
                    if ('script' === param.destination) {
                        CatalogController.reset();
                        CleaningController.reset();
                        ProcedureController.reset();
                        SeminarController.reset();

                        /*
                        alles außer Spracheinstellungen und Zustimmunginfo
                        löschen
                        */
                        LocalStorage.flush([
                            Config.storage.hasApproved,
                            Config.storage.languageCode,
                            Config.storage.languageData,
                            Config.storage.revision
                        ]);
                    }

                    $.mobile.changePage($(Config.pages.home));
                    $('#loading-overlay').fadeOut(hideSplashScreen);
                }
            });

            $('[rel=external]').on(Config.eventType, function(e) {
                e.preventDefault();
                window.open(this.href, '_system');
                Config.refreshState(this);
            });

            $(Config.scanner.home).on(Config.eventType, function(e) {
                e.preventDefault();
                Scanner.scan(isDevice);
                Config.refreshState(this);
            });

            $(Config.pages.terms + ', ' + Config.pages.imprint).on('pagebeforeshow', function() {
                iframes = iframes || initIFrames();
            });

            $(document).on({

                backbutton : function(e) {
                    e.preventDefault();
                    if ($.mobile.activePage.is(Config.pages.home)){
                        navigator.notification.confirm(
                            strings.exit,
                            function (buttonIndex) {
                                if (1 === buttonIndex) {
                                    // Ja, also App schließen
                                    $(document).off('backbutton');
                                    navigator.app.exitApp();
                                }
                            },
                            strings.hint,
                            strings.ok + ',' + strings.cancel
                        );
                    } else {
                        AppHistory.back();
                    }
                },

                cAlert : function(e, param) {
                    // log('cAlert', e, param);
                    try {
                        navigator.notification.alert(
                            strings[param].content,
                            null,
                            strings[param].header,
                            strings.ok
                        );
                    } catch(e) {
                        alert(strings[param].content);
                    }
                },

                cShowProduct : function(e, product) {
                    // log('cShowProduct', e, product);
                    ProductDetailController.initialise(product);
                },

                cShowSeminar : function(e, id) {
                    // log('cShowSeminar', e, id);
                    SeminarDetailController.initialise(id);
                },

                pagebeforechange : function(e, ui) {
                    // log('pagebeforechange', ui);
                    if ('object' === typeof ui.toPage) {
                        var page = '#' + $(ui.toPage).attr('id'),
                            prevent = false;
                        switch (page) {
                        case Config.pages.catalog:
                            if (!CatalogController.isready()) {
                                CatalogController.initialise(true);
                                prevent = true;
                            } else {
                                if ('#' + ui.options.fromPage[0].id !== Config.pages.filter) {
                                    CatalogController.resetFilter();
                                }
                            }
                            break;
                        case Config.pages.cleaning1:
                            if (!CleaningController.isready()) {
                                CleaningController.initialise();
                                prevent = true;
                            }
                            break;
                        case Config.pages.procedure1:
                            if (!ProcedureController.isready()) {
                                ProcedureController.initialise();
                                prevent = true;
                            }
                            break;
                        case Config.pages.language:
                            if (!isApprove) {
                                initLanguageSwitch();
                            }
                            break;
                        case Config.pages.seminars:
                            if (!SeminarController.isready()) {
                                SeminarController.initialise();
                                prevent = true;
                            }
                            break;
                        case Config.pages.request:
                            TechnicalRequest.initialise();
                            break;
                        case Config.pages.consultant:
                            Consultants.initialise();
                            break;
                        default:
                            break;
                        }

                        if (Config.pages.language !== page || !isApprove) {
                            AppHistory.push(page);
                        }

                        if (prevent) {
                            e.preventDefault();
                        }
                    }
                },

                pagechange : function(e, ui) {
                    // log('pagechange', ui);
                    switch ('#' + ui.toPage[0].id) {
                    case Config.pages.language:
                        $('#loading-overlay').fadeOut(hideSplashScreen);
                        break;
                    default:
                        break;
                    }
                }

            });

        }

        /**
        Android und iOS verfügen über einen SplashScreen, der nach erfolgreichem
        Laden geschlossen werden muss.

        @private
        @method hideSplashScreen
        **/
        function hideSplashScreen() {
            if (navigator.splashscreen) {
                navigator.splashscreen.hide();
            }
        }

        /**
        Die Höhe der iframes muss einmalig an die jeweilige sichtbare Höhe
        angepasst werden.

        @private
        @method initIFrames
        @return {Boolean} true Initialisierung stattgefunden
        **/
        function initIFrames() {
            $('iframe').css('height', ($(window).height() - 53) + "px");

            return true;
        }

        /**
        Zustimmung der gewerblichen Nutzung und Haftungsausschluss. Es
        werden sowohl das Template als auch die Events entsprechend
        initialisiert.

        @private
        @initApprovingDialog
        **/
        function initApprovingDialog() {

            try {
                navigator.notification.confirm(
                    strings.start_disclaimer,
                    function (buttonIndex) {
                        if (1 === buttonIndex) {
                            // Zustimmung erteilt — setze Flag im localStorage und
                            // leite auf den Homescreen weiter
                            isApprove = false;
                            LocalStorage.set(Config.storage.hasApproved, true);
                            $.mobile.changePage($(Config.pages.home));
                        } else {
                            // Zustimmung verweigert — lösche den localStorage und
                            // schließe idealerweise die App
                            LocalStorage.flush();
                        }
                    },
                    strings.start_approve,
                    strings.start_agree + ',' + strings.start_reject
                );
            } catch(e) {
                log(e);
                if (confirm(strings.start_disclaimer)) {
                    isApprove = false;
                    LocalStorage.set(Config.storage.hasApproved, true);
                    $.mobile.changePage($(Config.pages.home));
                }
            }

        }

        /**
        Initialisiere das Language-Template, um eine Sprachauswahl vorzunehmen.
        Nach erfolgter Sprachauswahl wird erst mal die Sprache geladen, bevor
        es weiter geht.

        @private
        @method initLanguageSwitch
        @param {Boolean} isStart findet die Sprachwahl beim Starten der App statt
        **/
        function initLanguageSwitch(isStart) {

            var _template = $(Config.pages.language),
                _header = _template.find('.header, h1');

            if (isStart || (!isApprove && _header.filter('.is-default').hasClass('is-hidden'))) {
                _header.toggleClass('is-hidden');
            }

            // Sicher stellen, dass der Sprachdialog nicht schon initialisiert wurde
            if (_template.data('is-inited')) {
                return;
            }
            log('Sprachdialog initialisieren');
            _template.data('is-inited', true);

            var _language = LocalStorage.get(Config.storage.languageCode),
                _links = $('.app-languages a', _template),
                _current = _links.filter('[data-value=' + _language + ']'),
                _more = $('.app-language-more', _template);

            if (_current.length) {
                _current.addClass('is-active');
            } else {
                _more.addClass('ui-disabled');
            }

            if (!isStart) {
                _more.hide();
            }

            _links.on(Config.eventType, function(e) {
                _more.removeClass('ui-disabled');
                _links.removeClass('is-active');
                $(this).addClass('is-active');
                if (!isStart) {
                    LanguageController.check($(this).data('value'));
                }
            });

            _more.on(Config.eventType, function(e) {
                var _active = _links.filter('.is-active');
                e.stopPropagation();
                e.preventDefault();
                LanguageController.check($(_active).data('value'));
            });

        }

        return {
            initialise : init
        };

    })();

    return App;
});
