define(['jquery', 'Config', 'LocalStorage', 'ExternalDataHandler', 'TemplateEngine'],
        function($, Config, LocalStorage, ExternalDataHandler, TemplateEngine) {

    'use strict';

    log('Modul CatalogController geladen!');

    /**
    Module-Pattern, da wir den CatalogController nur einmal benötigen. Das Modul
    kümmert sich um das Laden und Speichern des Produktkatalogs

    @class CatalogController
    @constructor
    **/
    var CatalogController = (function() {

        var isready  = false,    // true, wenn das Markup erstellt wurde
            catalog  = null,     // Katalogdaten für die Session
            listener = false,    // Listener nur ein einziges Mal initialisieren
            list,                // Referenz auf die Liste cachen
            listItems,           // gerenderte Items cachen
            // ist die App online, dann einmalig die Katalogdaten aktualisieren
            update = false;

        /**
        Initialisiere die Katalogdaten. Hole die Daten vom Server
        ab. Ist keine Online-Verbindung vorhanden, nutze die Daten aus dem
        localStorage, falls Daten vorhanden sind.

        @method initialise
        @param {Boolean} markup Markup nur dann bauen, wenn auch der Katalog
            angezeigt werden soll
        **/
        function init(markup) {
            if (update) {
                setCatalog(LocalStorage.get(Config.storage.catalogData), true, markup);
            } else {
                if (navigator.onLine) {
                    load(markup);    // Produktkatalog aktualisieren
                } else {
                    if (LocalStorage.get(Config.storage.catalogData)) {
                        setCatalog(LocalStorage.get(Config.storage.catalogData), true, markup);
                    } else {
                        // App muss online sein, um den Produktkatalog zu laden
                        $(document).trigger('cAlert', 'offline');
                        return;
                    }
                }
            }

            listener = listener || initListener();
        }

        /**
        Die Links im Panel fungieren als Filter für die Produktliste. Daher
        werden die Click- bzw. Touchevents entsprechend abgefangen und die
        Produktliste gefiltert.

        @private
        @method initListener
        @return {Boolean} true wenn die Listener initialisiert wurden
        **/
        function initListener() {
            $(document).on(Config.eventType, Config.pages.filter + ' [data-template=filterview] li', function(e) {
                e.preventDefault();
                var category = $(this).data('value');
                setFilter(category);
                log('Kategorie', category);
            });

            $(document).on(Config.eventType, Config.pages.catalog + ' [data-template=listview] .is-created', function(e) {
                e.preventDefault();
                var id = $(this).data('value');
                $(document).trigger('cShowProduct', {
                    productId : id,
                    origin    : Config.pages.catalog
                });
            });

            $('#custom-search input').on('keyup', function() {
                var input = $('.ui-input-text').not(this);
                input.val($(this).val()).change();
            });

            return true;
        }

        /**
        Die Links in dem Panel fungieren als Filter für die Katalogdaten und
        entsprechen den einzelnen Produktbereichen. Es werden lediglich die
        entsprechenden Produkte angezeigt und alle anderen Produkte
        ausgeblendet.

        @private
        @method setFilter
        @param {Number} category der anzuzeigende Produktbereich
        **/
        function setFilter(category) {
            if (-1 !== category) {
                resetFilter();

                var fragment = listItems.clone();

                if (category) {
                    list.before('<h2 class="head">' + catalog.cat['_' + category].name + '</h2>');
                    fragment = fragment.filter('[data-category=' + category + ']');
                }

                list.html(fragment);
                list.listview('refresh');
                if (category) {
                    $('.ui-li-divider', list).remove();
                }
            }

            $.mobile.changePage($(Config.pages.catalog), {
                transition : 'slide',
                reverse : true
            });
        }

        /**
        Der Filter wird zurück gesetzt. Dies geschieht z. B. dann, wenn die
        Produktlist erneut aufgerufen wird.

        @method resetFilter
        @param {Boolean} noTrigger
        **/
        function resetFilter() {
            $('input', Config.pages.catalog).val('').change();
            list.prev('h2').remove();
            list.html(listItems).listview('refresh');
        }

        /**
        Private Getter, der von der Public-API nach außen gereicht wird.

        @private
        @method getIsReady
        @return {Boolean} isready
        **/
        function getIsReady() {
            return isready;
        }

        /**
        Wurde das Markup noch nicht erstellt, so muss sowohl das Panel als
        auch die Liste selbst aus den Katalogdaten erstellt werden. Danach kann
        die Produktübersicht angezeigt werden.

        @private
        @method checkMarkup
        **/
        function checkMarkup() {
            if (!isready) {
                buildPanel();
                buildList();
                isready = true;
            } else {
                resetFilter();
            }
            Config.loader('hide');
            $.mobile.changePage($(Config.pages.catalog));
        }

        /**
        Die Liste aller Artikel kann mittels **catalog.articles** von den
        Katalogdaten erstellt werden.

        @example

            // Handlebars Template
            {{#each this}}
              <li class="is-created" data-value="{{id}}" data-category="{{cat}}">
                <a href="#detail-tab1">
                  <h2>{{name}}</h2>
                  <p>{{description}}</p>
                </a>
              </li>
            {{/each}}

        @private
        @method buildList
        **/
        function buildList() {
            var markup = TemplateEngine.render(Config.templates.catalogList, catalog.articles);
            list = $('[data-template=listview]', Config.pages.catalog);

            list.append(markup);
            if (list.hasClass('ui-listview')) {
                list.listview('refresh');
            }

            listItems = $('.is-created', list);
        }

        /**
        Die Produktbereiche werden als Filter verwendet. Diese sind in den
        Katalogdaten enthalten. Zugreifen kann man mittels **catalog.cat**

        @private
        @method buildPanel
        **/
        function buildPanel() {
            var container = $('[data-template=filterview]', Config.pages.filter),
                markup = TemplateEngine.render(Config.templates.catalogPanel, catalog.cat);

            container.append(markup);
            if (container.hasClass('ui-listview')) {
                container.listview('refresh');
            }
        }

        /**
        Wird eine neue Sprache geladen, dann muss auch das Markup auf den
        ursprünglichen Zustand zurück gesetzt werden. Dazu werden alle Nodes
        entfernt, die während der Markup-Erstellung mittels CSS-Klasse
        *is-created* markiert wurden.

        @private
        @method resetMarkup
        **/
        function resetMarkup() {
            var _collection = $(Config.pages.catalog + ',' + Config.pages.filter).find('.is-created');
            $(_collection).remove();
            isready = false;
        }

        /**
        Hole die Daten vom Server.

        @private
        @method load
        @param {Boolean} markup wird kein Markup benötigt, muss die Info
            durchgeschliffen werden
        **/
        function load(markup) {
            if (markup) {
                Config.loader('show');
            }

            var lang = LocalStorage.get(Config.storage.languageCode),
                path = Config.externalPaths.catalog.replace(/\{lang\}/, lang),

                getData = new ExternalDataHandler(Config.baseUrl + path, function(data) {
                    log('CatalogController - load()', data);
                    setCatalog(data, false, markup);
                }, true).process();
        }

        /**
        Wird die Sprache gewechselt, müssen die Daten erneut vom Server
        abgerufen werden. Also wird sowohl der localStorage als auch das
        Modul zurück gesetzt.

        @method reset
        **/
        function reset() {
            catalog = null;
            listItems = null;
            resetMarkup();
            update = false;
        }

        /**
        Speichere die Katalogdaten im LocalStorage.

        @private
        @method setCatalog
        @param {Object} data Produktkatalog
        @param {Boolean} local Daten kommen aus dem localStorage
        @param {Boolean} markup Markup nur erstellen, wenn auch die Produktliste
            angezeigt werden soll
        **/
        function setCatalog(data, local, markup) {
            if (!local) {
                LocalStorage.set(Config.storage.catalogData, data);
            }
            catalog = data;
            if (markup) {
                checkMarkup();
            }
        }

        /**
        Der Barcode-Scanner wie auch die Reinigungsempfehlung sind abhängig
        von dem Produktkatalog. Es wird der Katalog zurück gegeben und falls
        noch nicht vorhanden vom Server geladen.

        @method getCatalog
        @return {Object} Katalogdaten
        **/
        function getCatalog() {
            if (!catalog) {
                init(false);
            }
            return catalog;
        }

        /**
        @return public API
        **/
        return {
            getCatalog   : getCatalog,
            initialise   : init,
            initListener : initListener,
            isready      : getIsReady,
            reset        : reset,
            resetFilter  : resetFilter
        };

    })();

    return CatalogController;

});
