define(['jquery', 'Config', 'CatalogController', 'LocalStorage', 'ExternalDataHandler',
        'TemplateEngine'], function($, Config, CatalogController, LocalStorage,
        ExternalDataHandler, TemplateEngine) {

    'use strict';

    log('Modul CleaningController geladen!');

    /**
    Module-Pattern, da wir den CleaningController nur einmal benötigen. Er
    kümmert sich um das Laden und Speichern der Reinigungsempfehlungen sowie 
    um das Rendern der entsprechenden Auswahllisten.

    @class CleaningController
    @constructor
    **/
    var CleaningController = (function() {

        var isready  = false,    // true, wenn das Markup erstellt wurde
            catalog  = null,     // wir benötigen für die Liste den Produktkatalog
            cleaning = null,     // Reinigungsempfehlungsdaten für die Session
            listener = false,    // Listener nur ein einziges Mal initialisieren
            steps = {
                step1 : null,
                step2 : null
            },
            // ist die App online, dann einmalig die Reinigungsempfehlungen
            // aktualisieren
            update = false,

            /**
            Initialisiere die Reinigunsgempfehlung. Hole die Daten vom Server
            ab. Ist keine Online-Verbindung vorhanden, nutze die Daten aus dem
            localStorage, falls Daten vorhanden sind.

            @method initialise
            **/
            init = function() {
                if (update) {
                    setCleaning(LocalStorage.get(Config.storage.cleaningData), true);
                } else {
                    if (navigator.onLine) {
                        load();    // Reinigungsempfehlung aktualisieren
                    } else {
                        if (LocalStorage.get(Config.storage.cleaningData)) {
                            setCleaning(LocalStorage.get(Config.storage.cleaningData), true);
                        } else {
                            // App muss online sein, um die Reinigungsempfehlungen zu laden
                            $(document).trigger('cAlert', 'offline');
                            return;
                        }
                    }
                }

                CatalogController.initialise(false);
                listener = listener || initListener();
            },

            /**
            Die einzelnen Steps werden per Link angesteuert. Die Click- bzw.
            Touchevents müssen abgefangen werden, um die nächsten Steps abhängig
            von der Auswahl zu rendern.

            @method initListener
            @return {Boolean} true wenn die Listener initialisiert wurden
            **/
            initListener = function() {
                $(document).on(Config.eventType, Config.pages.cleaning1 + ' .is-created', function() {
                    // e.preventDefault();
                    steps.step1 = $(this).text();
                    buildStep2(cleaning.activity[$(this).data('value')].advises);
                });

                $(document).on(Config.eventType, Config.pages.cleaning2 + ' .is-created', function(e) {
                    // e.preventDefault();
                    steps.step2 = $(this).text();
                    catalog = CatalogController.getCatalog();
                    if (!catalog) {
                        // Der Katalog scheint noch nicht geladen zu sein
                        e.preventDefault();
                        $.mobile.changePage($(Config.pages.error));
                    } else {
                        buildStep3($(this).data('value'), $(this).data('hint'));
                    }
                });

                $(document).on(Config.eventType, Config.pages.cleaning3 + ' .is-created', function(e) {
                    e.preventDefault();
                    var id = $(this).data('value');
                    $(document).trigger('cShowProduct', {
                        productId : id,
                        origin    : Config.pages.cleaning3
                    });
                });

                return true;
            },

            /**
            Private Getter, der von der Public-API nach außen gereicht wird.

            @private
            @method getIsReady
            @return {Boolean} isready
            **/
            getIsReady = function() {
                return isready;
            },

            /**
            Wurde das Markup noch nicht erstellt, so muss sowohl das Panel als
            auch die Liste selbst aus den Katalogdaten erstellt werden. Danach kann
            die Produktübersicht angezeigt werden.

            @private
            @method checkMarkup
            **/
            checkMarkup = function() {
                if (!isready) {
                    buildStep1();
                    isready = true;
                }
                Config.loader('hide');
                $.mobile.changePage($(Config.pages.cleaning1));
            },

            /**
            Erstelle die Liste der Reinigungsempfehlungen. Diese sind enthalten
            in den Daten unter **cleaning.activity**.

            @method buildStep1
            **/
            buildStep1 = function() {
                var container = $('[data-template=listview]', Config.pages.cleaning1),
                    markup = TemplateEngine.render(Config.templates.cleaning1, cleaning.activity);

                container.append(markup);
                if (container.hasClass('ui-listview')) {
                    container.listview('refresh');
                }

            },

            /**
            Erstelle die Liste der Materialien. Dazu können die Materialien von
            den Daten mittels **cleaning.material** extrahiert werden.

            @method buildStep2
            @param {Object} data Objekt mit sämtlichen Materialien
            **/
            buildStep2 = function(data) {

                var container = $('[data-template=listview]', Config.pages.cleaning2),
                    materials = [],
                    markup = '',
                    i;

                // Reset, um neu zu bauen
                $('.is-created', container).remove();

                $('.cleaning-step1 span', Config.pages.cleaning2).text(steps.step1);

                /*
                Vorbereiten der Daten für das Rendern des Templates.
                Achtung: kein Array, sondern ein Objekt, daher for-in loop
                */
                for (i in data) {
                    if (data.hasOwnProperty(i)) {
                        materials.push({
                            hint     : data[i].hint,
                            material : cleaning.material[i],
                            products : data[i].products
                        });
                    }
                }

                markup = TemplateEngine.render(Config.templates.cleaning2, materials);

                container.append(markup);
                if (container.hasClass('ui-listview')) {
                    container.listview('refresh');
                }
            },

            /**
            Erstelle die Produktliste inkl. der Zusatzinformation. Dazu ist es
            notwendig, dass der Produktkatalog komplett und erfolgreich geladen
            wurde.

            @method buildStep3
            @param {Array} ids Liste aller zugehörigen Produkt-Ids
            @param {String} hint Zusatzinformation zu dem Material
            **/
            buildStep3 = function(ids, hint) {

                var container = $('[data-template=listview]', Config.pages.cleaning3),
                    articles = [],
                    data = ('string' === typeof ids) ? ids.split(',') : [ids],
                    markup = '',
                    i = 0,
                    l = data.length;

                // Reset, um neu zu bauen
                $('.is-created', container).remove();

                $('.cleaning-step1 span', Config.pages.cleaning3).text(steps.step1);
                $('.cleaning-step2 span', Config.pages.cleaning3).text(steps.step2);
                if (hint) {
                    $('h2.hint span:last', Config.pages.cleaning3).text(hint);
                    $('h2.hint', Config.pages.cleaning3).show();
                } else {
                    $('h2.hint', Config.pages.cleaning3).hide();
                }

                /*
                Vorbereiten der Daten für das Rendern des Templates.
                Hinweis: Die Ids der Artikel in der Produktliste werden zwar als
                Key verwendet, jedoch muss ein **_** als Prefix verwendet werden,
                da Chrome und somit auch Webkit ansonsten das Objekt automatisch
                neu und damit falsch sortiert.
                */
                for (; i < l; i += 1) {
                    if (catalog.articles['_' + data[i]]) {
                        articles.push(catalog.articles['_' + data[i]]);
                    }
                }

                markup = TemplateEngine.render(Config.templates.catalogList, articles);

                container.append(markup);
                if (container.hasClass('ui-listview')) {
                    container.listview('refresh');
                }

            },

            /**
            Hole die Daten vom Server.

            @private
            @method load
            **/
            load = function() {
                Config.loader('show');

                var lang = LocalStorage.get(Config.storage.languageCode),
                    path = Config.externalPaths.cleaning.replace(/\{lang\}/, lang),

                    getData = new ExternalDataHandler(Config.baseUrl + path, function(data) {
                        log('CleaningController - load()', data);
                        setCleaning(data);
                    }, true, function() {
                        $.mobile.changePage($(Config.pages.error));
                    }).process();
            },

            /**
            Wird eine neue Sprache geladen, dann muss auch das Markup auf den
            ursprünglichen Zustand zurück gesetzt werden. Dazu werden alle Nodes
            entfernt, die während der Markup-Erstellung mittels CSS-Klasse
            *is-created* markiert wurden.

            @private
            @method resetMarkup
            **/
            resetMarkup = function() {
                var _collection = $(Config.pages.cleaning1).find('.is-created');
                $(_collection).remove();
                isready = false;
            },

            /**
            Wird die Sprache gewechselt, müssen die Daten erneut vom Server
            abgerufen werden. Also wird sowohl der localStorage als auch das
            Modul zurück gesetzt.

            @method reset
            **/
            reset = function() {
                LocalStorage.rm(Config.storage.cleaningData);
                cleaning = null;
                resetMarkup();
                update = false;
            },

            /**
            Speichere die Reinigunsempfehlung im LocalStorage.

            @private
            @method setCleaning
            @param {Object} data Daten für die Reinigungsempfehlung
            @param {Boolean} local Daten stammen aus dem localStorage
            **/
            setCleaning = function(data, local) {
                if (!local) {
                    LocalStorage.set(Config.storage.cleaningData, data);
                }
                cleaning = data;
                checkMarkup();
            };

        /**
        @return public API
        **/
        return {
            initialise  : init,
            isready     : getIsReady,
            reset       : reset
        };

    })();

    return CleaningController;

});
