define(['jquery', 'Config', 'ExternalDataHandler', 'TemplateEngine', 'LocalStorage'],
        function($, Config, ExternalDataHandler, TemplateEngine, LocalStorage) {

    'use strict';

    log('Modul Consultants geladen!');

    /**
    Holt eine Liste aller Berater in der Nähe und listet diese auf.

    @class Consultants
    @constructor
    **/
    var Consultants = (function() {

        var consultant,
            inited = false,
            form = $(Config.pages.consultant + ' form');

        /**
        Initialisiere das Formular.

        @method init
        **/
        function init() {
            inited = inited || initForm();
            initOptions();
        }

        /**
        Listener zur Formularverarbeitung initialisieren.

        @private
        @method initForm
        @return {Boolean} true Initialisierung hat stattgefunden
        **/
        function initForm() {
            var sendData = {},
                formData,
                i,
                l;

            form.submit(function(e) {
                e.preventDefault();
                if (checkValues()) {
                    formData = form.serializeArray();
                    for (i = 0, l = formData.length; i < l; i += 1) {
                        sendData[formData[i].name] = formData[i].value;
                    }
                    load(sendData);
                }
            });

            return true;
        }

        /**
        Vom Server die Länder holen.

        @private
        @method initOptions
        **/
        function initOptions() {
            var lang = LocalStorage.get(Config.storage.languageCode),
                path = Config.externalPaths.countries.replace(/\{lang\}/, lang);

            new ExternalDataHandler(Config.baseUrl + path, function(data) {
                log('Consultants - initOptions()', data);
                buildOptions(data);
            }, true).process();

        }

        /**
        Die Auswahl-Optionen neu rendern.

        @private
        @method buildOptions
        @param {Object} countries Länder für die Optionsauswahl
        **/
        function buildOptions(countries) {
            var markup = TemplateEngine.render(Config.templates.countries, countries);

            $('#country').html(markup).selectmenu("refresh");
        }

        /**
        Es wird geprüft, ob alle Formularfelder richtig ausgefüllt wurden.

        @private
        @method checkValues
        @return {Boolean} sind alle Felder ausgefüllt oder nicht
        **/
        function checkValues() {
            var items = $('[data-required]', form),
                item,
                clean = true;

            items.each(function() {

                item = $(this);

                if (item.val().length) {
                    item.removeClass('error');
                } else {
                    clean = false;
                    if (!item.hasClass('error')) {
                        item.addClass('error');
                    }
                }
            });

            return clean;
        }

        /**
        Der Berater wird gerendert.

        @private
        @method buildMarkup
        **/
        function buildMarkup() {
            log('Consultants.buildMarkup', consultant);
            var container = $('[data-template=content]', Config.pages.consultant),
                markup = '';

            container.empty();

            markup = TemplateEngine.render(Config.templates.consultants, consultant);

            container.append(markup);
            // initContact();
            $('#addContact').on(Config.eventType, function(evt) {
                evt.preventDefault();
                addContact();
            });

            $(Config.pages.consultant).trigger('create');
            Config.loader('hide');
        }

        /**
        Sofern für den Berater noch kein Kontakt vorliegt, erzeuge einen neuen.

        @private
        @method initContact
        **/
        function initContact() {
            var options;

            try {
                options = new ContactFindOptions();
                options.filter = consultant.name;
                navigator.contacts.find(['displayName', 'nickname'], onFindSuccess, onFindError, options);
            } catch(e) {}
        }

        /**
        Erzeuge einen neuen Kontakt im Telefonbuch des Users.

        @private
        @method addContact
        **/
        function addContact() {
            var contact = navigator.contacts.create(),
                name = new ContactName(),
                organization = new ContactOrganization(),
                tmpName = consultant.name.split(' ');

            $('#addContact').addClass('ui-disabled');

            name.givenName = tmpName.shift();
            name.familyName = tmpName.join(' ');

            contact.name = name;
            contact.displayName = consultant.name;
            contact.nickname = consultant.name;     //specify both to support all devices

            contact.phoneNumbers = [new ContactField('work', consultant.telefon, true)];

            contact.emails = [new ContactField('work', consultant.email, true)];

            organization.name = 'KIEHL Group';
            contact.organizations = [organization];

            contact.save(onSaveSuccess, onSaveError);
        }

        /**
        Callback für die Kontaksuche. Wird ein Kontakt gefunden, so wird der
        Button zum Speichern ausgeblendet.

        @private
        @method onFindSuccess
        @param {Array} contacts gefundene Kontakte
        **/
        function onFindSuccess(contacts) {
            if (contacts.length) {
                $('#addContact').addClass('ui-disabled');
            } else {
                $('#addContact').on(Config.eventType, addContact);
            }
        }

        /**
        Callback für die Kontaktsuche. Geht bei der Suche etwas schief, wird
        zumindest der Button initialisiert.

        @private
        @method onFindError
        @param {Object} contactError Fehlerobjekt
        **/
        function onFindError(contactError) {
            $('#addContact').on(Config.eventType, addContact);
        }

        /**
        Callback für das Speichern eines Kontaktes. Wurde der Kontakt erfolgreich
        gespeichert, dann wird ein Notification angezeigt und der Button
        ausgeblendet.

        @private
        @method onSaveSuccess
        **/
        function onSaveSuccess() {
            $(document).trigger('cAlert', 'adviser_success');
        }

        /**
        Callback für das Speichern eines Kontaktes. Geht beim Speichern etwas
        schief, dann wird lediglich eine Notification angezeigt.

        @private
        @method onSaveError
        @param {Object} contactError Fehlerobjekt
        **/
        function onSaveError() {
            $(document).trigger('cAlert', 'adviser_error');
        }

        /**
        Hole die Daten vom Server.

        @private
        @method load
        @param {Object} location Formulardaten
        **/
        function load(location) {
            Config.loader('show');

            var lang = LocalStorage.get(Config.storage.languageCode),
                path = Config.externalPaths.consultant.replace(/\{lang\}/, lang).
                        replace(/\{country\}/, location.country).
                        replace(/\{zip\}/, location.zip),

                getData = new ExternalDataHandler(Config.baseUrl + path, function(data) {
                    log('Consultants - load()', data);
                    setConsultantData(data);
                }, true).process();

        }

        /**
        Nutze die Daten und rendere eine Liste aller gefundenen Berater.

        @private
        @method setConsultantData
        @param {Object} data Daten der Berater
        **/
        function setConsultantData(data) {
            consultant = data;
            buildMarkup();
        }

        /**
        @return Public API
        **/
        return {
            initialise : init
        };

    }());

    return Consultants;

});
