define(['jquery', 'Config'], function($, Config) {

    'use strict';

    log('Modul CoverageController geladen!');

    /**
    Der Verbrauchsrechner ist ein losgelöstes Modul der Produktdetailansicht.

    @class CoverageController
    @constructor
    **/
    var CoverageController = (function() {

        var inited = false,
            page,    // die Seite für den Verbrauchsrechner cachen
            use,
            application;

        /**
        Der Verbrauchsrechner ist neu zu initialisieren. Das übergebene Objekt
        entspricht folgender JSON-Struktur:

            {
                "filename" : "http:\/\/<host>\/images\/icons\/picto01.gif",
                "pdfname"  : "http:\/\/<host>\/images\/download\/picto_32.pdf",
                "headline" : "Polieren:",
                "copytext" : "Die abgetrocknete Pflegeschicht kann durch Polieren …",
                "consumption" : "180",
                "consumption_unit" : "ml"
            }

        Die H1 der Seite wird schon durch den ProduktDetailController befüllt.

        @method init
        @param {Object} consumption das Anwendungs-Objekt
        **/
        function init(consumption) {
            log('Verbrauchsrechner für ', consumption);
            use = parseFloat(consumption.consumption);

            application = consumption;
            inited = inited || initPage();

            $('.head b', Config.pages.coverage).text(application.headline);
            $('#consumption b').text(application.consumption_unit);
            $('[data-translate=detail_area]', Config.pages.coverage).text(application.consumption_ref || page.ref);

            page.input.text('0');
            page.result.text('0');
            $('#surface').trigger('click');
        }

        /**
        Um den gesamten Platz zu nutzen, der zur Verfügung steht, muss dem
        Container für den Rechner eine Höhe gegeben werden. Darüber hinaus werden
        die Listener definiert.

        @private
        @method initPage
        @return {Boolean} true bedeutet Initialisierung ist abgeschlossen
        **/
        function initPage() {
            log(Config);

            page = {
                button  : $('.grid-3 > div', Config.pages.coverage),
                context : $('#coverage-content', Config.pages.coverage),
                goal    : $('.grid-2 > div', Config.pages.coverage),
                input   : $('#surface p', Config.pages.coverage),
                ref     : $('[data-translate=detail_area]', Config.pages.coverage).text(),
                result  : $('#consumption p', Config.pages.coverage),
                reverse : false    // bestimmt die Richtung der Berechnung
            };

            // initial fix - anzunehmender Zwischenwert
            page.context.height($(window).height() - 150);

            $(Config.pages.coverage).on('pageshow', function() {
                var height = $(window).height() - page.context.offset().top;
                page.context.height(height);
            });

            page.button.each(function() {
                this.addEventListener(('ontouchstart' in document.documentElement) ? 'touchstart' : 'click', function(e) {
                    e.stopPropagation();
                    changeValues($(this).data('value'));
                }, false);
            });

            page.goal.on(Config.eventType, function(e) {
                e.preventDefault();
                if (!$(this).hasClass('is-active')) {
                    page.goal.toggleClass('is-active');
                    page.input = $(this).find('p');
                    page.result = $(this).siblings().find('p');
                    page.reverse = !page.reverse;
                }
            });

            return true;
        }

        /**
        Wird ein Button gedrückt, dann muss sowohl der eingegebene Wert wie auch
        der berechnete Wert neu ermittelt werden.

        @private
        @method changeValues
        @param {String} value Wert des gedrückten Buttons
        @return {Null} falls nichts zu tun ist bei z. B. Komma schon gesetzt
        **/
        function changeValues(value) {
            log('Button "' + value + '" tapped');
            var oldValue = page.input.text(),
                addComma = false,
                comma = -1 !== oldValue.indexOf(','),
                newValue,
                result,
                tmpValue;

            // steht eine 0 und wird die 0 gedrückt, dann nix tun
            if ('0' === oldValue && 0 === value) {
                return;
            }

            oldValue = oldValue.length && '0' !== oldValue ? oldValue : '';

            if ('EN' === Config.currentLanguage) {
                oldValue = oldValue.replace(/,/g, '');
                comma = -1 !== oldValue.indexOf('.');
            } else {
                oldValue = oldValue.replace(/\./g, '').replace(',', '.');
            }

            if (10 > value) {
                // Ziffer
                newValue = oldValue + '' + value;
            } else if (10 === value) {
                // Komma
                if (comma) {
                    return;     // Komma schon gesetzt, also nix weiter zu tun
                }
                newValue = ('' === oldValue ? '0' : oldValue);
                addComma = true;
            } else {
                // Back, letztes Zeichen abschneiden
                newValue = oldValue.replace(/(.$)/, '');
            }

            tmpValue = newValue;
            newValue = parseFloat(newValue.length ? newValue : '0');

            if (page.reverse) {
                result = newValue / use;
            } else {
                result = newValue * use;
            }

            page.input.text(numberFormat(newValue, true, addComma, tmpValue));
            page.result.text(numberFormat(result));
        }

        /**
        Die Ausgabe der Werte ist lokalisiert.

        @private
        @method numberFormat
        @param {String} value
        @param {Boolean} custom Custom-Format
        @param {Boolean} comma wird erstmalig ein Komma eingegeben, dann soll dies auch ausgegeben werden
        @return {String} lokalisierter Wert
        **/
        function numberFormat(value, custom, comma, rawValue) {
            var rounded = Math.floor(value),
                decimal = custom || value === rounded ? 0 : 2,
                decimalSep = ',',
                thousandsSep = '.',
                output;

            // Sonderfall Italienisch berücksichtigen
            if ('EN' === Config.currentLanguage) {
                decimalSep = '.';
                thousandsSep = ',';
            }


            if (custom) {
                output = number_format(rounded, decimal, decimalSep, thousandsSep);
                output += comma ? decimalSep : '';

                if (rawValue && -1 !== rawValue.indexOf('.')) {
                    output += decimalSep + rawValue.split('.')[1];
                }
            } else {
                output = number_format(value, decimal, decimalSep, thousandsSep);
            }

            return output;

        }

        /**
        Portierung der PHP-Funktion number_format()

        @private
        @method number_format
        @param {Mixed} number
        @param {Number} decimals Anzahl der Nachkommastellen
        @param {String} decimalSep Dezimal-Trennzeichen
        @param {String} thousandsSep Tausender-Trennzeichen
        @return {String} formatierte Nummer
        **/
        function number_format(number, decimals, decimalSep, thousandsSep) {

            number = (number + '').replace(/[^0-9+\-Ee.]/g, '');
        
            var n = !isFinite(+number) ? 0 : +number,
                prec = !isFinite(+decimals) ? 0 : Math.abs(decimals),
                sep = (typeof thousandsSep === 'undefined') ? ',' : thousandsSep,
                dec = (typeof decimalSep === 'undefined') ? '.' : decimalSep,
                s = '',
                toFixedFix = function (n, prec) {
                    var k = Math.pow(10, prec);
                    return '' + Math.round(n * k) / k;
                };

            // Fix for IE parseFloat(0.55).toFixed(0) = 0;
            s = (prec ? toFixedFix(n, prec) : '' + Math.round(n)).split('.');
            if (s[0].length > 3) {
                s[0] = s[0].replace(/\B(?=(?:\d{3})+(?!\d))/g, sep);
            }
            if ((s[1] || '').length < prec) {
                s[1] = s[1] || '';
                s[1] += new Array(prec - s[1].length + 1).join('0');
            }
            return s.join(dec);
        }

        /**
        @return Public-API
        **/
        return {
            initialise : init
        };

    }());

    return CoverageController;

});
