define(['jquery'], function($) {

    'use strict';

    log('Modul ExternalDataHandler geladen');

    /**
    Der ExternalDataHandler kümmert sich um das holen und bereitstellen von Daten
    von internern Resourcen und von externen Systemen per JSON bzw. JSONP.

    @class ExternalDataHandler
    @constructor
    @param {String} url externe URL
    @param {Object} success Funktion, der im Erfolgsfall das JSON-Objekt zurück gegeben wird
    @param {Boolean} external extern muss JSONP sprechen
    @param {Object} error Funktion, die bei Fehlern aufgerufen wird
    **/
    var ExternalDataHandler = function(url, success, external, error) {

        var _callbacks = ExternalDataHandler.callbacks,
            _destroyed = false,
            _error = error || _callbacks.error,
            _external = external,
            _success = success || _callbacks.success,
            _url = url;

        /**
        JSON-Daten mittels jQuery.getJSON() holen und dem Callback das geparste
        Objekt übergeben.

        @private
        @method getData
        **/
        function getData() {
            if (_external) {
                _url += '&jsonp=?';
            }
            $.ajax({
                dataType : 'json',
                url : _url,
                success : function(data) {
                    log('ExternalDataHandler - getData()', data);
                    if (!_destroyed) {
                        _success(data);
                    }
                },
                error : function(error) {
                    log('ExternalDataHandler - ERROR: ' + error);
                    if (_error && !_destroyed) {
                        _error();
                    }
                },
                timeout : 10000
            });
        }

        /**
        Ist ein Request noch nicht fertig verarbeitet und wird in der App jedoch
        schon weiter navigiert, kann mittels der Funktion verhindert werden, dass
        die Callbacks aufgerufen werden. Somit kann ein fehlerhaftes Verhalten bzw.
        Nebeneffekte vermieden werden.

        @method destroy
        **/
        this.destroy = function() {
            _destroyed = true;
        };

        /**
        Daten laden, aber nur wenn auch ein Callback für den Erfolgsfall definiert
        ist, da der Request ansonsten sinnlos ist.

        @method process
        **/
        this.process = function() {
            if (!_success) {
                log('Ups, kein Callback definiert! Dann braucht\'s auch kein Request');
                return;
            }

            getData();
        };

        // Chaining erlauben
        return this;

    };

    /**
    Globale Callbacks des Handlers. Somit kann eine global einheitliche
    Fehlerbehandlung gewährleistet werden. Darüber hinaus kann auch ein
    allgemein gültiger Callback für den Erfolgsfall bestimmt werden.

    @static
    @property callbacks
    @for ExternalDataHandler
    **/
    ExternalDataHandler.callbacks = {
        error   : null,
        success : null
    };

    return ExternalDataHandler;

});
