define(['jquery', 'Config', 'Languages', 'LocalStorage'],
        function($, Config, Languages, LocalStorage) {

    'use strict';

    log('Modul LanguageController geladen!');

    /**
    Module-Pattern, da wir den LanguageController nur einmal benötigen.

    @class LanguageController
    @constructor
    **/
    var LanguageController = (function() {
        var strings,

            /**
            Wurde eine Sprache ausgewählt, dann werden die Zeichenketten aus dem
            localStorage geholt. Falls dieser leer sein sollte, dann wird das
            Sprachpaket von dem globalen Objekt **Languages** extrahiert.

            @method check
            @param {String} langcode die zu ladende Sprache
            **/
            check = function(langcode) {
                var oldCode = LocalStorage.get(Config.storage.languageCode),
                    newCode = (langcode || 'de').toUpperCase();    // Fallback berücksichtigen

                if (Config.revision !== parseInt(LocalStorage.get(Config.storage.revision), 10)) {
                    // Sprachpaket neu laden
                    oldCode = null;
                    LocalStorage.set(Config.storage.revision, Config.revision);
                }

                if (!oldCode || oldCode !== newCode) {
                    setLanguage(newCode, Languages[newCode]);
                } else {
                    setStrings(LocalStorage.get(Config.storage.languageData), 'localStorage', oldCode);
                }
            },

            /**
            Sprachwahl und Sprachdatei im LocalStorage speichern und die Zeichenketten
            bereit stellen.

            @private
            @method setLanguage
            @param {String} lang Sprachcode
            @param {String} data Inhalt der geladenen Sprachdatei
            **/
            setLanguage = function (lang, data) {
                LocalStorage.set(Config.storage.languageCode, lang);
                LocalStorage.set(Config.storage.languageData, data);
                setStrings(data, 'script', lang);
            },

            /**
            Speichere die geholten Daten in einer privaten Variable und werfe
            ein CustomEvent, um z. B. das initiale Parsen des DOM anzustoßen

            @private
            @method setStrings
            @param {Object} data das geladene Sprachpaket
            @param {String} dest Info, aus welcher Quelle die Sprachdatei geladen wurde
            @param {String} lang Language-Code
            **/
            setStrings = function(data, dest, lang) {
                Config.currentLanguage = lang;

                strings = data;
                $(document).trigger('cLanguageLoaded', {
                    destination : dest
                });
            },

            /**
            Private Getter, der über die public-API lediglich den Zugriff auf die
            geladenen Zeichenketten erlaubt

            @method getStrings
            @return {object} Sprachpaket
            **/
            getStrings = function() {
                return strings;
            };

        /**
        @return public API
        **/
        return {
            check      : check,
            getStrings : getStrings
        };

    })();

    return LanguageController;

});