define([
        'jquery',
        'Config',
        'LocalStorage'
    ], function($, Config, LocalStorage) {

    'use strict';

    /**
     * PDFs sind an verschiedenen Stellen in der App zu öffnen. Neben der
     * Produktdetailseite sind auch bei den Reinigungsverfahren PDFs
     * hinterlegt.
     *
     * @class PDFDialog
     * @constructor
    **/
    var PDFDialog = (function() {

        var language;

        /**
         * Links zu externen PDFs werden je nach verwendetem Betriebssystem
         * unterschiedlich behandelt.
         *
         * @method handle
         * @param  {Object} lang  aktuelles Sprachpaket
         * @param  {String} uri   URL des Dokumentes
         * @param  {[type]} title Titel des Dokumentes
         * @return {null}
         */
        function handle(lang, uri, title) {
            var useFallback = false,
                url = encodeURI(uri);

            language = lang;
            title = title || language.hint;

            log('Öffne PDF: ' + url);

            switch (Config.platform) {
                case 'Android':
                    showPdfDialog(url, title);
                    break;
                case 'iOS':
                    try {
                        navigator.notification.confirm(
                                '',
                                function(buttonIndex) {
                                    if (2 === buttonIndex) {
                                        // öffnen mit...
                                        window.plugins.socialsharing.share(
                                            null,
                                            null,
                                            url
                                        );
                                        // ExternalFileUtil.openWith(url, 'com.adobe.pdf');
                                    } else {
                                        window.open(url, '_blank', 'location=no,enableViewportScale=yes');
                                    }
                                },
                                title,
                                language.show + ',' + language.open
                                );
                    } catch (e) {
                        useFallback = true;
                    }
                    break;
                default:
                    useFallback = true;
            }

            if (useFallback) {
                window.open(url, '_blank', 'location=no');
            }
        }

        /**
         * In den aktuellsten API-Versionen geht pdf.js nicht mehr, was dazu führt,
         * dass wir direkt den System-Dialog öffnen. Dabei wird noch eine
         * Notification dem User angezeigt, indem darauf hingewiesen wird, dass
         * ein PDF-Viewer auf dem Gerät installiert sein muss.
         *
         * @private
         * @method showPdfDialog
         * @param {String} url URL des Dokumentes
         * @param {String} title Titel des Dokumentes
         * @return {Null}
         */
        function showPdfDialog(url, title) {
            if (LocalStorage.get(Config.storage.pdfHint)) {
                window.open(url, '_system');
            } else {
                navigator.notification.confirm(
                    language.pdf.hint,
                    function (buttonIndex) {
                        if (2 === buttonIndex) {
                            // Hinweis nicht mehr anzeigen und PDF anzeigen
                            LocalStorage.set(Config.storage.pdfHint, true);
                        }

                        // Standardfall PDF anzeigen
                        window.open(url, '_system');
                    },
                    language.hint,
                    language.pdf.view + ',' + language.pdf.hide
                );
            }
        }

        /**
         @return public API
         **/
        return {
            handle : handle
        };

    }());

    return PDFDialog;

});
