define([
        'jquery',
        'Config',
        'LocalStorage',
        'ExternalDataHandler',
        'PDFDialog',
        'TemplateEngine',
        'LanguageController'
    ], function(
        $,
        Config,
        LocalStorage,
        ExternalDataHandler,
        PDFDialog,
        TemplateEngine,
        LanguageController
    ) {

    'use strict';

    log('Modul ProcedureController geladen!');

    /**
    Module-Pattern,
    da wir den ProcedureController nur einmal benötigen. Er
    kümmert sich um das Laden und Speichern der Reinigungsverfahren sowie
    um das Rendern der entsprechenden Auswahllisten.

    @class ProcedureController
    @constructor
    **/
    var ProcedureController = (function() {

        var isready  = false,       // true, wenn das Markup erstellt wurde
            procedure = null,       // Reinigungsempfehlungsdaten für die Session
            language,               // aktuelle Sprache
            listener = false,       // Listener nur ein einziges Mal initialisieren
            steps = {
                step1 : null,
                step2 : null
            },
            // ist die App online, dann einmalig die Reinigungsempfehlungen
            // aktualisieren
            update = false,

            /**
            Initialisiere die Reinigunsgempfehlung. Hole die Daten vom Server
            ab. Ist keine Online-Verbindung vorhanden, nutze die Daten aus dem
            localStorage, falls Daten vorhanden sind.

            @method initialise
            **/
            init = function() {
                if (update) {
                    setProcedure(LocalStorage.get(Config.storage.procedureData), true);
                } else {
                    if (navigator.onLine) {
                        load();    // Reinigungsempfehlung aktualisieren
                    } else {
                        if (LocalStorage.get(Config.storage.procedureData)) {
                            setProcedure(LocalStorage.get(Config.storage.procedureData), true);
                        } else {
                            // App muss online sein, um die Reinigungsempfehlungen zu laden
                            $(document).trigger('cAlert', 'offline');
                            return;
                        }
                    }
                }

                listener = listener || initListener();
            },

            /**
            Die einzelnen Steps werden per Link angesteuert. Die Click- bzw.
            Touchevents müssen abgefangen werden, um die nächsten Steps abhängig
            von der Auswahl zu rendern.

            @method initListener
            @return {Boolean} true wenn die Listener initialisiert wurden
            **/
            initListener = function() {
                $(document).on(Config.eventType, Config.pages.procedure1 + ' .is-created', function() {
                    // e.preventDefault();
                    steps.step1 = $(this).text();
                    buildStep2(procedure.procedure[$(this).data('value')].files);
                });

                $(document).on(Config.eventType, Config.pages.procedure2 + ' a[href*=".pdf"]', function(e) {
                    e.preventDefault();
                    PDFDialog.handle(language, this.href);
                });

                return true;
            },

            /**
            Private Getter, der von der Public-API nach außen gereicht wird.

            @private
            @method getIsReady
            @return {Boolean} isready
            **/
            getIsReady = function() {
                return isready;
            },

            /**
            Wurde das Markup noch nicht erstellt, so muss sowohl das Panel als
            auch die Liste selbst aus den Katalogdaten erstellt werden. Danach kann
            die Produktübersicht angezeigt werden.

            @private
            @method checkMarkup
            **/
            checkMarkup = function() {
                if (!isready) {
                    buildStep1();
                    isready = true;
                }
                Config.loader('hide');
                $.mobile.changePage($(Config.pages.procedure1));
            },

            /**
            Erstelle die Liste der Reinigungsempfehlungen. Diese sind enthalten
            in den Daten unter **procedure.procedure**.

            @method buildStep1
            **/
            buildStep1 = function() {
                var container = $('[data-template=listview]', Config.pages.procedure1),
                    markup = TemplateEngine.render(Config.templates.procedure1, procedure.procedure);

                container.append(markup);
                if (container.hasClass('ui-listview')) {
                    container.listview('refresh');
                }

            },

            /**
            Erstelle die Liste der Materialien. Dazu können die Materialien von
            den Daten mittels **procedure.material** extrahiert werden.

            @method buildStep2
            @param {Object} data Objekt mit sämtlichen Materialien
            **/
            buildStep2 = function(data) {

                var container = $('[data-template=listview]', Config.pages.procedure2),
                    materials = [],
                    markup = '',
                    i;

                // Reset, um neu zu bauen
                $('.is-created', container).remove();

                $('.procedure-step1 span', Config.pages.procedure2).text(steps.step1);

                /*
                Vorbereiten der Daten für das Rendern des Templates.
                Achtung: kein Array, sondern ein Objekt, daher for-in loop
                */
                for (i in data) {
                    if (data.hasOwnProperty(i)) {
                        materials.push({
                            material : procedure.material[i],
                            link     : data[i]
                        });
                    }
                }

                markup = TemplateEngine.render(Config.templates.procedure2, materials);

                container.append(markup);
                if (container.hasClass('ui-listview')) {
                    container.listview('refresh');
                }
            },

            /**
            Hole die Daten vom Server.

            @private
            @method load
            **/
            load = function() {
                Config.loader('show');

                var lang = LocalStorage.get(Config.storage.languageCode),
                    path = Config.externalPaths.procedure.replace(/\{lang\}/, lang),

                    getData = new ExternalDataHandler(Config.baseUrl + path, function(data) {
                        log('ProcedureController - load()', data);
                        setProcedure(data);
                    }, true, function() {
                        $.mobile.changePage($(Config.pages.error));
                    }).process();
            },

            /**
            Wird eine neue Sprache geladen, dann muss auch das Markup auf den
            ursprünglichen Zustand zurück gesetzt werden. Dazu werden alle Nodes
            entfernt, die während der Markup-Erstellung mittels CSS-Klasse
            *is-created* markiert wurden.

            @private
            @method resetMarkup
            **/
            resetMarkup = function() {
                var _collection = $(Config.pages.procedure1).find('.is-created');
                $(_collection).remove();
                isready = false;
            },

            /**
            Wird die Sprache gewechselt, müssen die Daten erneut vom Server
            abgerufen werden. Also wird sowohl der localStorage als auch das
            Modul zurück gesetzt.

            @method reset
            **/
            reset = function() {
                LocalStorage.rm(Config.storage.procedureData);
                procedure = null;
                resetMarkup();
                update = false;
            },

            /**
            Speichere die Reinigunsempfehlung im LocalStorage.

            @private
            @method setProcedure
            @param {Object} data Daten für die Reinigungsempfehlung
            @param {Boolean} local Daten stammen aus dem localStorage
            **/
            setProcedure = function(data, local) {
                if (!local) {
                    LocalStorage.set(Config.storage.procedureData, data);
                }
                procedure = data;
                language = LanguageController.getStrings();
                checkMarkup();
            };

        /**
        @return public API
        **/
        return {
            initialise  : init,
            isready     : getIsReady,
            reset       : reset
        };

    })();

    return ProcedureController;

});
