define([
        'jquery',
        'Config',
        'LanguageController',
        'LocalStorage',
        'ExternalDataHandler',
        'TemplateEngine',
        'CoverageController',
        'PDFDialog'
    ], function($, Config, LanguageController, LocalStorage, ExternalDataHandler,
        TemplateEngine, CoverageController, PDFDialog) {

    'use strict';

    log('Modul ProductDetailController geladen!');

    /**
     Module-Pattern, da wir den ProductDetailController nur einmal benötigen. Das Modul
     kümmert sich um das Laden und Speichern der Produktdaten

     @class ProductDetailController
     @constructor
     **/
    var ProductDetailController = (function() {

        var product = null, // Produktdaten
            language,
            current = 0,
            inited = false,
            iframe = null,
            obj, // initiierendes Objekt
            tabs = [],
            tabsInited = false,
            // ist die App online, dann einmalig die Produktdaten aktualisieren
            update = false;

        /**
         Initialisiere die Katalogdaten. Hole die Daten vom Server
         ab. Ist keine Online-Verbindung vorhanden, nutze die Daten aus dem
         localStorage, falls Daten vorhanden sind.

         @method initialise
         @param {Objekt} eventObject Produkt-Id und Origin
         **/
        function init(eventObject) {
            obj = eventObject;

            Config.loader('show');

            var key = Config.storage.productPrefix + obj.productId;

            if (update) {
                setProductData(LocalStorage.get(key), true);
            } else {
                if (navigator.onLine) {
                    load();    // Produktdaten aktualisieren
                } else {
                    if (LocalStorage.get(key)) {
                        setProductData(LocalStorage.get(key), true);
                    } else {
                        // App muss online sein, um die Produktdaten zu laden
                        $(document).trigger('cAlert', 'offline');
                        Config.loader('hide');
                        return;
                    }
                }
            }

            inited = inited || initListener();
        }

        /**
        Integriere iframe für PDF-Viewer.

        @private
        @method initIframe
        **/
        function initIframe() {
            var container = $(Config.pages.pdf),
                wrapper = $('[data-role=content]', container);

            wrapper.css('padding', '0');
            iframe = $('<iframe width="100%" height="400" seamless></iframe>');
            iframe.css({
                height : ($(window).height() - 53) + 'px',
                width  : '100%'
            });

            wrapper.append(iframe);
        }

        /**
        Lade das externe PDF in dem iframe. Dazu wird pdf.js genutzt.

        @private
        @method show
        @param {String} url Pfad zum PDF
        @param {String} title Titel der PDF-Datei
        **/
        // function showPDF(url, title) {
        //     var src = Config.localPaths.pdfjs;

        //     if (!iframe) {
        //         initIframe();
        //     }

        //     $('.share-button', Config.pages.pdf).attr('href', url).attr('title', title);

        //     iframe.attr('src', src + encodeURI(url));
        //     $.mobile.changePage($(Config.pages.pdf), {
        //         transition : 'slideup'
        //     });

        // }

        /**
         * In den aktuellsten API-Versionen geht pdf.js nicht mehr, was dazu führt,
         * dass wir direkt den Share-Dialog öffnen.
         *
         * @private
         * @method showShareDialog
         * @param {String} url URL des Dokumentes
         * @param {String} title Titel des Dokumentes
         * @return {Null}
         */
        function showShareDialog(url, title) {
            try {
                window.plugins.share.show(
                    {
                        subject: title,
                        text: url
                    },
                    function() {
                    // Erfolg - wir machen nix
                    },
                    function() {
                        // Fehler - zumindest vom Browser öffnen
                        window.open(url, '_system', 'location=no');
                    }
                );
            } catch(e) {
                window.open(url, '_system', 'location=no');
            }

        }

        /**
         Es werden die Listener initialisiert, aber nur einmalig. Links zu PDF-
         Dateien werden abhängig von der Plattform unterschiedlich behandelt. Auf
         iOS kann gewählt werden zwischen dem internen WebView mit integriertem
         PDF-Viewer und bei Android wird der native Teilen-Dialog hergenommen. Per
         Default-Wert wird das PDF mit einem neuen ChildBrowser bzw. in einem neuen
         Fenster geöffnet.

         @private
         @method initListener
         @return {Boolean} true Listener initialisiert
         **/
        function initListener() {

            $(document).on(Config.eventType, Config.pages.detail2 + ' a[href="#coverage"]', function() {
                var data = product.coverage[$(this).data('value')];
                $(Config.pages.coverage + ' h1').text(product.name);
                CoverageController.initialise(data);
            });

            $(document).on(Config.eventType, Config.pages.detail3 + ' .click-button', function(e) {
                e.preventDefault();
                // toggle panel from bottom to top
                $(this).toggleClass('is-active');
                toggleSlider();
            });

            $(document).on(Config.eventType, Config.pages.pdf + ' .share-button', function(e) {
                var title = this.title,
                    url = this.href;

                e.preventDefault();

                showShareDialog(url, title);

                Config.refreshState(this);
            });

            $(document).on(Config.eventType, Config.pages.detail3 + ' a[href*=".pdf"]', function(e) {
                var useFallback = false,
                    // url = encodeURI(this.href),
                    title = $(this).text();

                this.blur();
                e.preventDefault();

                PDFDialog.handle(language, this.href, title);
            });

            return true;
        }

        /**
         Die einzelnen Tabs werden über Events der Navbar gesteuert. Wird initial
         in die Detailseite gesprungen (neues Produkt), so landet man auf der
         Übersichtsseite. Der Rücksprung vom Verbrauchsrechner hingegen darf keinen
         Wechsel erzwingen.

         @private
         @method initTabs
         **/
        function initTabs() {
            var links = $('[data-role=navbar] a', Config.pages.details);

            if (tabsInited) {
                // Reset
                links.removeClass('is-active');
                links.get(0).classList.add('is-active');
                $(Config.pages.detail2 + ', ' + Config.pages.detail3).hide();
                $(Config.pages.detail1).show();
                current = 0;
                $(Config.pages.details).css({
                    overflow: 'visible'
                });
            } else {
                links.each(function(index) {
                    var self = this;
                    $(self).on(Config.eventType, function() {
                        if (index !== current) {
                            links.get(current).classList.remove('is-active');
                            tabs[current].fadeOut(150, function() {
                                tabs[index].fadeIn(150);
                                current = index;
                                $(Config.pages.details).css({
                                    overflow: 2 === current ? 'hidden' : 'visible'
                                });
                            });
                            this.classList.add('is-active');
                        }
                    });
                });
                tabsInited = true;
            }
        }

        /**
         Die Produktdetails werden gerendert.

         @private
         @method buildMarkup
         **/
        function buildMarkup() {
            log('ProductDetailController.buildMarkup', product);
            buildTabOverview();
            buildTabApplication();
            buildTabDocs();
            initTabs();
            $(Config.pages.details).trigger('create');

            if (obj.origin) {
                $('.header-back', Config.pages.details).attr('href', obj.origin);
            }
        }

        /**
         Tab 1 der Produktdetailseite rendern.

         @private
         @method buildTabOverview
         **/
        function buildTabOverview() {
            var container = $(Config.pages.detail1),
                    markup = '';

            if (!tabsInited) {
                tabs.push(container);
            }

            $('h2', Config.pages.details).html(product.name);

            container.empty();

            markup = TemplateEngine.render(Config.templates.detail1, product);

            container.append(markup);
            $.mobile.changePage($(Config.pages.details));
            Config.loader('hide');
        }

        /**
         Tab 2 der Produktdetailseite rendern.

         @private
         @method buildTabApplication
         **/
        function buildTabApplication() {
            var container = $(Config.pages.detail2),
                    markup = '';

            if (!tabsInited) {
                tabs.push(container);
            }

            $('h2', Config.pages.detail2).html(product.name);

            container.empty();

            markup = TemplateEngine.render(Config.templates.detail2, product);

            container.append(markup);
        }

        /**
         Tab 3 der Produktdetailseite rendern.

         @private
         @method buildTabDocs
         **/
        function buildTabDocs() {
            var container = $(Config.pages.detail3),
                    langcode = LocalStorage.get(Config.storage.languageCode),
                    markup = '';

            if (!tabsInited) {
                tabs.push(container);
            }

            $('h2', Config.pages.detail3).html(product.name);

            container.empty();
            // Produkt um Sprachdetail erweitern
            product.currentLanguage = {
                code: langcode,
                name: product.languages[langcode]
            };

            markup = TemplateEngine.render(Config.templates.detail3, product);

            container.append(markup);
            $('.dok ul', container).not('.' + langcode).hide();

            // Hm, anscheinend zickt auch die Phone-Variante rum. Also auch hier
            // auf pageshow verzichten und mit zwei Sekunden Verzögerung arbeiten.
            setTimeout(initSlider, 2000);

        }

        /**
        Slider initialisieren. Der Listener auf den Button wird per delegate
        abgefangen.

        @private
        @method initSlider
        **/
        function initSlider() {
            var container = $(Config.pages.detail3),
                o = {};

            // Full Box Sliding (Hidden to Visible) with css-transition
            o.box          = $('.panel-block');
            o.dokBoxHeight = $('.dok').outerHeight(true);
            o.button       = $('.click-button');
            o.slider       = $('.slider');
            o.buttonHeight = o.button.css('height');
            o.logoHeight   = $('.logo', container).outerHeight(true);
            o.header       = o.logoHeight;

            o.navH2        = $('.navbar-blue h2', container).outerHeight(true);
            o.navUl        = $('.ui-grid-b li a.ui-btn-active', container).outerHeight(true);

            o.nav          = $('.ui-navbar', container).outerHeight(true);
            o.windowHeight = $(window).height() - 60;
            o.boxHeight    = o.windowHeight - o.header - o.nav - o.dokBoxHeight;
            // height auto for box

            $('.language-list li', o.slider).on(Config.eventType, function(e) {
                if (e.preventBubble) { e.preventBubble(); }
                if (e.preventDefault) { e.preventDefault(); }
                if (e.stopPropagation) { e.stopPropagation(); }
                if (e.stopImmediatePropagation) { e.stopImmediatePropagation(); }
                $('.dok ul', container).show();
                $('.dok ul', container).not('.' + $(this).data('value')).hide();
                $('.icons-language', o.button).attr('class', 'icons-language ' + $(this).data('value'));
                $('.language-name', o.button).text(product.languages[$(this).data('value')]);
                toggleSlider();
            });
        }

        /**
        Slider togglen.

        @private
        @method toggleSlider
        **/
        function toggleSlider() {
            var slider = $(Config.pages.detail3 + ' .slider');

            slider.toggleClass('is-open-slider');
        }

        /**
         Hole die Daten vom Server.

         @private
         @method load
         @param {Number} id ID des zu ladenden Produkts
         **/
        function load() {
            var lang = LocalStorage.get(Config.storage.languageCode),
                    path = Config.externalPaths.product.replace(/\{lang\}/, lang),
                    param = 'id=' + (obj.productId || 0),
                    getData;

            if (obj.ean) {
                param = 'ean=' + obj.ean;
            }

            path = path.replace(/\{product\}/, param);

            getData = new ExternalDataHandler(Config.baseUrl + path, function(data) {
                log('ProductDetailController - load()', data);
                setProductData(data, false);
            }, true).process();
        }

        /**
         Speichere die Katalogdaten im LocalStorage. Wird per Barcodescanner ein
         Produkt angefordert, so kann nicht lokal gespeichert werden, da die
         Produkt-Id nicht vorliegt, sondern per gescanntem EAN-Code geladen wird.

         @private
         @method setProductData
         @param {Object} data Produktdaten
         @param {Number} id ID des Produkts
         @param {Boolean} local Daten kommen aus dem localStorage
         **/
        function setProductData(data, local) {
            if (!data) {
                $(document).trigger('cAlert', 'product_notfound');
                Config.loader('hide');
                return;
            }

            if (!local && obj.productId) {
                LocalStorage.set(Config.storage.productPrefix + obj.productId, data);
            }

            product = data;
            language = LanguageController.getStrings();
            buildMarkup();
        }

        /**
         @return public API
         **/
        return {
            initialise: init,
        };

    })();

    return ProductDetailController;

});
