define(['jquery', 'Config', 'LocalStorage', 'ExternalDataHandler', 'TemplateEngine'],
        function($, Config, LocalStorage, ExternalDataHandler, TemplateEngine) {

    'use strict';

    log('Modul SeminarController geladen!');

    /**
    Module-Pattern, da wir den SeminarController nur einmal benötigen. Das Modul
    kümmert sich um das Laden und Speichern der Seminardaten
    
    @class SeminarController
    @constructor
    **/
    var SeminarController = (function() {

        var isready  = false,    // true, wenn das Markup erstellt wurde
            seminars = null,     // Seminardaten für die Session
            listener = false,    // Listener nur ein einziges Mal initialisieren
            // ist die App online, dann einmalig die Seminardaten aktualisieren
            update = false,

            /**
            Initialisiere die Katalogdaten. Hole die Daten vom Server
            ab. Ist keine Online-Verbindung vorhanden, nutze die Daten aus dem
            localStorage, falls Daten vorhanden sind.

            @method initialise
            **/
            init = function() {
                if (update) {
                    setSeminarData(LocalStorage.get(Config.storage.seminarData), true);
                } else {
                    if (navigator.onLine) {
                        load();    // Seminardaten aktualisieren
                    } else {
                        if (LocalStorage.get(Config.storage.seminarData)) {
                            setSeminarData(LocalStorage.get(Config.storage.seminarData), true);
                        } else {
                            // App muss online sein, um den Seminardaten zu laden
                            $(document).trigger('cAlert', 'offline');
                            return;
                        }
                    }
                }

                listener = listener || initListener();
            },

            /**
            Wird ein Seminar geklickt, dann wird ein CustomEvent gefeuert, um die
            Detailansicht entsprechend befüllen zu können.

            @method initListener
            @return {Boolean} true wenn die Listener initialisiert wurden
            **/
            initListener = function() {
                $(document).on(Config.eventType, Config.pages.seminars + ' [data-template=listview] .is-created', function(e) {
                    e.preventDefault();
                    var id = $(this).data('value');
                    $(document).trigger('cShowSeminar', id);
                });

                return true;
            },

            /**
            Private Getter, der von der Public-API nach außen gereicht wird.

            @private
            @method getIsReady
            @return {Boolean} isready
            **/
            getIsReady = function() {
                return isready;
            },

            /**
            Wurde das Markup noch nicht erstellt, so muss sowohl das Panel als
            auch die Liste selbst aus den Katalogdaten erstellt werden. Danach kann
            die Produktübersicht angezeigt werden.

            @private
            @method checkMarkup
            **/
            checkMarkup = function() {
                if (!isready) {
                    buildList();
                    isready = true;
                }
                Config.loader('hide');
                $.mobile.changePage($(Config.pages.seminars));
            },

            /**
            Die Liste aller Seminare wird gerendert.

            @private
            @method buildList
            **/
            buildList = function() {
                var container = $('[data-template=listview]', Config.pages.seminars),
                    markup = TemplateEngine.render(Config.templates.seminarList, seminars);

                container.html(markup);
                if (container.hasClass('ui-listview')) {
                    container.listview('refresh');
                }
            },

            /**
            Wird eine neue Sprache geladen, dann muss auch das Markup auf den
            ursprünglichen Zustand zurück gesetzt werden. Dazu werden alle Nodes
            entfernt, die während der Markup-Erstellung mittels CSS-Klasse
            *is-created* markiert wurden.

            @private
            @method resetMarkup
            **/
            resetMarkup = function() {
                var _collection = $(Config.pages.seminars).find('.is-created');
                $(_collection).remove();
                isready = false;
            },

            /**
            Hole die Daten vom Server.

            @private
            @method load
            **/
            load = function() {
                Config.loader('show');

                var lang = LocalStorage.get(Config.storage.languageCode),
                    path = Config.externalPaths.seminars.replace(/\{lang\}/, lang),

                    getData = new ExternalDataHandler(Config.baseUrl + path, function(data) {
                        log('SeminarController - load()', data);
                        setSeminarData(data, false);
                    }, true).process();
            },

            /**
            Wird die Sprache gewechselt, müssen die Daten erneut vom Server
            abgerufen werden. Also wird sowohl der localStorage als auch das
            Modul zurück gesetzt.

            @method reset
            **/
            reset = function() {
                seminars = null;
                resetMarkup();
                update = false;
            },

            /**
            Speichere die Katalogdaten im LocalStorage.

            @private
            @method setSeminarData
            @param {Object} data Seminardaten
            @param {Boolean} local Daten kommen aus dem localStorage
            **/
            setSeminarData = function(data, local) {
                if (!local) {
                    LocalStorage.set(Config.storage.seminarData, data);
                }
                seminars = data;
                checkMarkup();
            };

        /**
        @return public API
        **/
        return {
            initialise   : init,
            isready      : getIsReady,
            reset        : reset
        };

    })();

    return SeminarController;

});
