define(['jquery', 'Config', 'LanguageController', 'LocalStorage', 'ExternalDataHandler',
        'TemplateEngine'], function($, Config, LanguageController, LocalStorage,
        ExternalDataHandler, TemplateEngine) {

    'use strict';

    log('Modul SeminarDetailController geladen!');

    /**
    Module-Pattern, da wir den SeminarController nur einmal benötigen. Das Modul
    kümmert sich um das Laden und Speichern der Seminardaten
    
    @class SeminarController
    @constructor
    **/
    var SeminarController = (function() {

        var inited = false,
            seminar = null,    // Seminardaten
            language,
            // ist die App online, dann einmalig die Seminardaten aktualisieren
            update = false;

        /**
        Initialisiere die Katalogdaten. Hole die Daten vom Server
        ab. Ist keine Online-Verbindung vorhanden, nutze die Daten aus dem
        localStorage, falls Daten vorhanden sind.

        @method initialise
        @param {Number} id Seminar-Id
        **/
        function init(id) {
            Config.loader('show');

            var key = Config.storage.seminarPrefix + id;

            if (update) {
                setSeminarData(LocalStorage.get(key), true);
            } else {
                if (navigator.onLine) {
                    load(id);    // Seminardaten aktualisieren
                } else {
                    if (LocalStorage.get(key)) {
                        setSeminarData(LocalStorage.get(key), true);
                    } else {
                        // App muss online sein, um die Seminardaten zu laden
                        $(document).trigger('cAlert', 'offline');
                        return;
                    }
                }
            }

            inited = inited || initListener();
        }

        /**
        Wird ein Seminar geklickt, dann muss der Inhalt neu gerendert werden, da
        die Termine als Button ausgegeben werden.

        @method initListener
        @return {Boolean} true wenn die Listener initialisiert wurden
        **/
        function initListener() {
            $(Config.pages.seminar).on('pagebeforeshow', function() {
                $(Config.pages.seminar).trigger('create');
            });

            return true;
        }

        /**
        Die Seminardetails werden gerendert.

        @private
        @method buildMarkup
        **/
        function buildMarkup() {
            log(seminar);
            var container = $('[data-template=content]', Config.pages.seminar),
                markup = '';

            container.empty();

            seminar.course_dates = language.course_dates;
            markup = TemplateEngine.render(Config.templates.seminar, seminar);

            container.append(markup);
            Config.loader('hide');
            $.mobile.changePage($(Config.pages.seminar));
        }

        /**
        Hole die Daten vom Server.

        @private
        @method load
        @param {Number} id ID des zu ladenden Seminars
        **/
        function load(id) {

            var lang = LocalStorage.get(Config.storage.languageCode),
                path = Config.externalPaths.seminar.replace(/\{lang\}/, lang).
                        replace(/\{seminar_id\}/, id),

                getData = new ExternalDataHandler(Config.baseUrl + path, function(data) {
                    log('SeminarController - load()', data);
                    setSeminarData(data, id, false);
                }, true).process();
        }

        /**
        Speichere die Katalogdaten im LocalStorage.

        @private
        @method setSeminarData
        @param {Object} data Seminardaten
        @param {Number} id ID des Seminars
        @param {Boolean} local Daten kommen aus dem localStorage
        **/
        function setSeminarData(data, id, local) {
            if (!local) {
                LocalStorage.set(Config.storage.seminarPrefix + id, data);
            }
            seminar = data;
            language = LanguageController.getStrings();
            buildMarkup();
        }

        /**
        @return public API
        **/
        return {
            initialise   : init,
        };

    })();

    return SeminarController;

});
