define(['jquery', 'Config', 'LocalStorage'], function($, Config, LocalStorage) {

    'use strict';

    log('Modul TechnicalRequest geladen!');

    /**
    Eine technische Anfrage verschicken.

    @class TechnicalRequest
    @constructor
    **/
    var TechnicalRequest = (function() {

        var form = $(Config.pages.request + ' form'),
            imageWrapper = document.getElementById('getPhotoWrapper'),
            image,
            inited = false,
            uri = null;

        /**
        Initialisiere den Listener, aber nur einmalig.

        @method init
        @return null wenn schon initialisiert
        **/
        function init() {
            inited = inited || initForm();
            reset();
        }

        /**
        Listener zur Formularverarbeitung initialisieren.

        @private
        @method initForm
        @return {Boolean} true Initialisierung hat stattgefunden
        **/
        function initForm() {
            form.submit(function(e) {
                e.preventDefault();
                if (checkValues()) {
                    sendValues();
                }
            });

            $('#getPhoto').on(Config.eventType, function() {
                capture();
            });

            imageWrapper.querySelector('a').addEventListener(Config.eventType, function(e) {
                e.preventDefault();
                resetImage();
            });

            return true;
        }

        /**
        Ein Foto mit der Kamera des Devices aufnehmen.

        @private
        @method capture
        **/
        function capture() {
            try {
                navigator.camera.getPicture(onCaptureSuccess, onCaptureFail, {
                    quality          : 70,
                    destinationType  : Camera.DestinationType.FILE_URI,
                    saveToPhotoAlbum : true
                });
            } catch(e) {
                log('Kamera nicht vorhanden oder anderer Fehler');
                triggerAlert('error');
            }
        }

        /**
        Wurde ein Bild aufgenommen bzw. ausgewählt, dann wird dieses in den
        dafür vorgesehenen Wrapper integriert und die URI für den Email-Versand
        gespeichert.

        @private
        @method onCaptureSuccess
        @private {URI} imageURI Bild als data-URI
        **/
        function onCaptureSuccess(imageURI) {
            resetImage();
            image = new Image();
            image.src = uri = imageURI;
            imageWrapper.appendChild(image);
            imageWrapper.classList.remove('is-hidden');
        }

        /**
        Schlägt die Fotoauswahl fehl, dann loggen wir das lediglich.

        @private
        @method onCaptureFail
        @message {String} message Fehlermeldung, weswegen der Capturevorgang fehlgeschlagen ist
        **/
        function onCaptureFail(message) {
            log('Failed because: ' + message);
            triggerAlert('error');
        }

        /**
        Wurden die Daten erfolgreich gesendet, dann muss das Formular geresettet
        und das Bild gelöscht werden.

        @private
        @method onPostSuccess
        **/
        function onPostSuccess() {
            reset();
            Config.loader('hide');
            triggerAlert('request_success');
        }

        /**
        Fehler an die Trigger-Funktion durchreichen.

        @private
        @method onPostError
        **/
        function onPostError() {
            Config.loader('hide');
            triggerAlert('error');
        }

        /**
        Der Request kann auch mal fehlschlagen. Also müssen wir auch in diesem
        Fall eine Notification werfen.

        @private
        @method triggerAlert
        @param {String} type Typ des Fehlers wird einfach mittels Event durchgereicht
        **/
        function triggerAlert(type) {
            $(document).trigger('cAlert', type);
        }

        /**
        Das komplette Formular muss beim Betreten der Seite oder nach dem
        Abschicken des Formulars zurück gesetzt werden.

        @private
        @method reset
        **/
        function reset() {
            form[0].reset();
            $('[data-required]', form).removeClass('error');
            resetImage();
        }

        /**
        Das Imagefeld zurücksetzen.

        @private
        @method resetImage
        @return {nothing} gibt es kein Bild, muss auch nichts zurück gesetzt werden
        **/
        function resetImage() {
            if (!image) {
                return;
            }
            $(image).remove();
            uri = null;
            imageWrapper.classList.toggle('is-hidden');
        }

        /**
        Die Daten werden versendet. Hierbei wird unterschieden, ob das Cordova-
        Plugin **EmailComposer** vorhanden ist. Wenn ja, dann wird dieses genutzt,
        um nicht die App zu verlassen, wenn nicht, wird das Formular per *mailto:*
        versendet.

        @private
        @method sendValues
        **/
        function sendValues() {
            var formData = form.serializeArray(),
                ft,
                i = 0,
                l = formData.length,
                options,
                submitData = {},
                url = Config.baseUrl + Config.externalPaths.request;

            for (; i < l; i += 1) {
                submitData[formData[i].name] = formData[i].value;
            }

            submitData.realm = Config.realm;
            submitData.lang  = LocalStorage.get(Config.storage.languageCode);

            log(submitData);

            if (navigator.onLine) {
                Config.loader('show');
                if (uri) {
                    options = new FileUploadOptions();
                    options.fileKey='image';
                    options.fileName=uri.substr(uri.lastIndexOf('/')+1);
                    options.mimeType='image/jpeg';

                    options.params = submitData;
                    options.chunkedMode = false;

                    ft = new FileTransfer();
                    ft.upload(uri, url, onPostSuccess, onPostError, options);
                } else {
                    $.ajax({
                        url : url,
                        data : submitData,
                        dataType : 'text',
                        timeout : 5000,
                        type : 'POST',
                        success : onPostSuccess,
                        error : onPostError
                    });
                }
            } else {
                triggerAlert('offline');
            }
        }

        /**
        Es wird geprüft, ob alle Formularfelder richtig ausgefüllt wurden.

        @private
        @method checkValues
        @return {Boolean} sind alle Felder ausgefüllt oder nicht
        **/
        function checkValues() {
            var items = $('[data-required]', form),
                item,
                clean = true;

            items.each(function() {

                item = $(this);

                if (item.val().length) {
                    item.removeClass('error');
                } else {
                    clean = false;
                    if (!item.hasClass('error')) {
                        item.addClass('error');
                    }
                }
            });

            return clean;
        }

        /**
        @return Public API
        **/
        return {
            initialise : init
        };

    }());

    return TechnicalRequest;

});
